@@ -2,7 +2,7 @@
 *     Exim - an Internet mail transport agent    *
 *************************************************/
 
-/* Copyright (c) University of Cambridge 1995 - 2004 */
+/* Copyright (c) University of Cambridge 1995 - 2009 */
 /* See the file NOTICE for conditions of use and distribution. */
 
 /* Private structure for the private options. */
@@ -21,18 +21,20 @@ typedef struct {
   uschar *quota_warn_threshold;
   uschar *mailbox_size_string;
   uschar *mailbox_filecount_string;
+  uschar *expand_maildir_use_size_file;
   uschar *maildir_dir_regex;
   uschar *maildir_tag;
+  uschar *maildirfolder_create_regex;
   uschar *mailstore_prefix;
   uschar *mailstore_suffix;
   uschar *check_string;
   uschar *escape_string;
   uschar *file_format;
-  int   mailbox_size_value;
+  off_t quota_value;
+  off_t quota_warn_threshold_value;
+  off_t mailbox_size_value;
   int   mailbox_filecount_value;
-  int   quota_value;
   int   quota_filecount_value;
-  int   quota_warn_threshold_value;
   int   mode;
   int   dirmode;
   int   lockfile_mode;
@@ -90,6 +92,6 @@ extern void appendfile_transport_init(transport_instance *);
 
 /* Function that is shared with tf_maildir.c */
 
-extern int  check_dir_size(uschar *, int *, const pcre *);
+extern off_t  check_dir_size(uschar *, int *, const pcre *);
 
 /* End of transports/appendfile.h */
